@@ -55,16 +55,14 @@
 //! let mut r = ring![1, 2, 3, 4, 5];
 //!
 //! r.rotate(1);
-//! assert_eq!(r.collect(), vec![5, 1, 2, 3, 4]);
+//! assert_eq!(r.collect_vec(), vec![5, 1, 2, 3, 4]);
 //!
 //! r.rotate(-2);
-//! assert_eq!(r.collect(), vec![2, 3, 4, 5, 1]);
+//! assert_eq!(r.collect_vec(), vec![2, 3, 4, 5, 1]);
 //! ```
 //!
 //! Ring implements `collect` to collect the elements in the ring as a vector
-//! if the type of the elements implements the `Copy` trait.
-//! It also implements `into_iter` to generate an iterator. However,
-//! `into_iter` empties the ring.
+//! if the type of the elements implement the `Copy` trait.
 //!
 //! ```rust
 //! # #[macro_use] extern crate ring_queue;
@@ -72,14 +70,37 @@
 //! use ring_queue::Ring;
 //!
 //! let mut r = ring![1, 2, 3, 4];
-//! assert_eq!(r.collect(), vec![1, 2, 3, 4]);
+//! assert_eq!(r.collect_vec(), vec![1, 2, 3, 4]);
+//! ```
+//!
+//! It also implements `into_iter` to generate an iterator. However,
+//! `into_iter` empties the ring unless the elements implement the `Copy` trait.
+//!
+//! ```rust
+//! # #[macro_use] extern crate ring_queue;
+//!
+//! use ring_queue::Ring;
+//!
+//! #[derive(Debug)]
+//! struct Foo { a: i32 }
+//!
+//! let mut r = ring![Foo{a: 1}, Foo{a: 2}];
 //! assert_eq!(r.is_empty(), false);
 //!
 //! for item in r.into_iter() {
-//!     println!("{}", item);
+//!     println!("{:?}", item);
 //! }
 //!
 //! assert_eq!(r.is_empty(), true);
+//!
+//! let r2 = ring![1, 2, 3, 4];
+//! assert_eq!(r2.is_empty(), false);
+//!
+//! for item in r2.into_iter() {
+//!     println!("{}", item);
+//! }
+//!
+//! assert_eq!(r2.is_empty(), false);
 //! ```
 
 #[derive(Copy, Clone)]
@@ -102,9 +123,11 @@ macro_rules! ring {
     () => ( $crate::Ring::new() );
     ($($x:expr),*) => (
         {
+            let s: Box<[_]>  = Box::new([$($x),*]);
+            let mut x = s.into_vec();
             let mut r = Ring::new();
-            for elem in [$($x),*].iter() {
-                r.push(*elem);
+            for i in x.drain(..) {
+                r.push(i);
             }
             r
         }
@@ -265,7 +288,7 @@ where
     ///
     /// let mut r = ring![1, 2, 3];
     /// r.push(4);
-    /// assert_eq!(r.collect(), vec![1, 2, 3, 4]);
+    /// assert_eq!(r.collect_vec(), vec![1, 2, 3, 4]);
     /// # }
     /// ```
     pub fn push(&mut self, value: T) {
@@ -284,7 +307,7 @@ where
     ///
     /// let mut r = ring![1, 2, 3];
     /// r.push_left(0);
-    /// assert_eq!(r.collect(), vec![0, 1, 2, 3]);
+    /// assert_eq!(r.collect_vec(), vec![0, 1, 2, 3]);
     /// # }
     /// ```
     pub fn push_left(&mut self, value: T) {
@@ -327,10 +350,10 @@ where
     /// let mut r = ring![1, 2, 3, 4, 5];
     ///
     /// r.rotate(1);
-    /// assert_eq!(r.collect(), vec![5, 1, 2, 3, 4]);
+    /// assert_eq!(r.collect_vec(), vec![5, 1, 2, 3, 4]);
     ///
     /// r.rotate(-2);
-    /// assert_eq!(r.collect(), vec![2, 3, 4, 5, 1]);
+    /// assert_eq!(r.collect_vec(), vec![2, 3, 4, 5, 1]);
     /// # }
     /// ```
     pub fn rotate(&mut self, n: isize) -> &mut Self {
@@ -343,27 +366,21 @@ where
     }
 
     #[inline]
-    fn peek_left(&self, cur: usize, n: isize) -> Option<T>
-    where
-        T: Copy,
-    {
+    fn peek_left(&self, cur: usize, n: isize) -> Option<&T> {
         let mut c = cur;
         for _ in n..0 {
             c = self.items[c].prev;
         }
-        self.items[c].val
+        self.items[c].val.as_ref()
     }
 
     #[inline]
-    fn peek_right(&self, cur: usize, n: isize) -> Option<T>
-    where
-        T: Copy,
-    {
+    fn peek_right(&self, cur: usize, n: isize) -> Option<&T> {
         let mut c = cur;
         for _ in 0..n {
             c = self.items[c].next;
         }
-        self.items[c].val
+        self.items[c].val.as_ref()
     }
 
     /// Retrieve the element `n` steps to the right from the head. If `n` is
@@ -379,15 +396,12 @@ where
     /// use ring_queue::Ring;
     ///
     /// let r = ring![1, 2, 3, 4, 5];
-    /// assert_eq!(r.peek(0), Some(1));
-    /// assert_eq!(r.peek(1), Some(2));
-    /// assert_eq!(r.peek(-1), Some(5));
+    /// assert_eq!(r.peek(0), Some(&1));
+    /// assert_eq!(r.peek(1), Some(&2));
+    /// assert_eq!(r.peek(-1), Some(&5));
     /// # }
     /// ```
-    pub fn peek(&self, n: isize) -> Option<T>
-    where
-        T: Copy,
-    {
+    pub fn peek(&self, n: isize) -> Option<&T> {
         match self.cur {
             Some(cur) => {
                 if n < 0 {
@@ -443,7 +457,7 @@ where
     ///
     /// let mut r = ring![1, 2, 3];
     /// r.append(&mut ring![4, 5, 6]);
-    /// assert_eq!(r.collect(), vec![1, 2, 3, 4, 5, 6]);
+    /// assert_eq!(r.collect_vec(), vec![1, 2, 3, 4, 5, 6]);
     /// # }
     /// ```
     pub fn append(&mut self, other: &mut Ring<T>) {
@@ -467,7 +481,7 @@ where
     ///
     /// let mut r = ring![4, 5, 6];
     /// r.append_left(&mut ring![3, 2, 1]);
-    /// assert_eq!(r.collect(), vec![1, 2, 3, 4, 5, 6]);
+    /// assert_eq!(r.collect_vec(), vec![1, 2, 3, 4, 5, 6]);
     /// # }
     /// ```
     pub fn append_left(&mut self, other: &mut Ring<T>) {
@@ -487,7 +501,7 @@ where
     ///
     /// let mut r = ring![1, 2, 3];
     /// r.reverse();
-    /// assert_eq!(r.collect(), vec![3, 2, 1]);
+    /// assert_eq!(r.collect_vec(), vec![3, 2, 1]);
     /// # }
     /// ```
     pub fn reverse(&mut self) {
@@ -534,10 +548,10 @@ where
     /// use ring_queue::Ring;
     ///
     /// let r = ring![1, 2, 3];
-    /// assert_eq!(r.collect(), vec![1, 2, 3]);
+    /// assert_eq!(r.collect_vec(), vec![1, 2, 3]);
     /// # }
     /// ```
-    pub fn collect(&self) -> Vec<T>
+    pub fn collect_vec(&self) -> Vec<T>
     where
         T: Copy,
     {
@@ -560,12 +574,12 @@ where
     }
 }
 
-impl<'a, T> std::iter::IntoIterator for &'a mut Ring<T> {
+impl<'a, T> std::iter::IntoIterator for &'a Ring<T> where T: Copy {
     type Item = T;
-    type IntoIter = Iter<T>;
+    type IntoIter = Iter<'a, T>;
 
-    /// Creates a consuming iterator, that is, one that moves each value out of
-    /// the ring (from head to tail). The ring will be empty.
+    /// Creates an iterator that will not consume the current ring. It will,
+    /// instead, copy the elements one by one to the iterator.
     ///
     /// # Examples
     ///
@@ -579,6 +593,41 @@ impl<'a, T> std::iter::IntoIterator for &'a mut Ring<T> {
     /// for i in r.into_iter() {
     ///     println!("{}", i);
     /// }
+    ///
+    /// // r is not empty now
+    /// ```
+    fn into_iter(self) -> Self::IntoIter {
+        Iter {
+            items: &self.items,
+            head: self.cur.clone(),
+            pos: self.cur.clone(),
+        }
+    }
+}
+
+impl<'a, T> std::iter::IntoIterator for &'a mut Ring<T> {
+    type Item = T;
+    type IntoIter = IterMut<T>;
+
+    /// Creates a consuming iterator, that is, one that moves each value out of
+    /// the ring (from head to tail). The ring will be empty.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use ring_queue::Ring;
+    ///
+    /// struct Foo { a: i32 }
+    ///
+    /// let mut r = Ring::new();
+    /// r.push(Foo{a: 1});
+    /// r.push(Foo{a: 2});
+    ///
+    /// for i in r.into_iter() {
+    ///     println!("{}", i.a);
+    /// }
+    ///
+    /// // r is empty now
     /// ```
     fn into_iter(self) -> Self::IntoIter {
         let mut items = Vec::new();
@@ -588,7 +637,7 @@ impl<'a, T> std::iter::IntoIterator for &'a mut Ring<T> {
         std::mem::swap(&mut cur, &mut self.cur);
         self.items.clear();
 
-        Iter {
+        IterMut {
             items: items,
             head: cur,
             pos: cur,
@@ -609,13 +658,38 @@ where
     }
 }
 
-pub struct Iter<T> {
+pub struct Iter<'a, T: Copy> {
+    items: &'a Vec<Item<T>>,
+    head: Option<usize>,
+    pos: Option<usize>,
+}
+
+impl<'a, T: Copy> Iterator for Iter<'a, T> {
+    type Item = T;
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let mut val = None;
+        self.pos = self.pos.and_then(|pos| {
+            let item = self.items[pos];
+            val = item.val;
+
+            if item.next == self.head.unwrap() {
+                None
+            } else {
+                Some(item.next)
+            }
+        });
+        val
+    }
+}
+
+pub struct IterMut<T> {
     items: Vec<Item<T>>,
     head: Option<usize>,
     pos: Option<usize>,
 }
 
-impl<T> Iterator for Iter<T> {
+impl<T> Iterator for IterMut<T> {
     type Item = T;
 
     fn next(&mut self) -> Option<Self::Item> {
@@ -676,6 +750,50 @@ impl<T: fmt::Debug + Copy> fmt::Debug for Ring<T> {
     }
 }
 
+impl<T> std::ops::Index<isize> for Ring<T> {
+    type Output = T;
+
+    fn index(&self, idx: isize) -> &T {
+        match self.peek(idx) {
+            Some(r) => r,
+            None => panic!("tried to access index {} of empty ring", idx),
+        }
+    }
+}
+
+impl<T: PartialEq> PartialEq for Ring<T> {
+    fn eq(&self, other: &Ring<T>) -> bool {
+        if self.len() != other.len() {
+            return false;
+        }
+
+        let cur = self.cur.unwrap();
+        let cur2 = other.cur.unwrap();
+        let mut next = cur;
+        let mut next2 = cur2;
+
+        loop {
+            let v = &self.items[next].val;
+            let v2 = &other.items[next2].val;
+            if v != v2 {
+                return false;
+            }
+
+            if next == cur && next2 == cur2 {
+                break;
+            } else if next == cur || next2 == cur2 {
+                return false;
+            }
+
+            next = self.items[next].next;
+            next2 = other.items[next2].next;
+        }
+        true
+    }
+}
+
+impl<T: Eq> Eq for Ring<T> {}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -686,10 +804,10 @@ mod tests {
         let mut r = ring![1, 2];
 
         r.push(3);
-        assert_eq!(r.collect(), vec![1, 2, 3]);
+        assert_eq!(r.collect_vec(), vec![1, 2, 3]);
 
         r.push_left(0);
-        assert_eq!(r.collect(), vec![0, 1, 2, 3]);
+        assert_eq!(r.collect_vec(), vec![0, 1, 2, 3]);
     }
 
     #[test]
@@ -699,48 +817,61 @@ mod tests {
     }
 
     #[test]
-    fn test_iter() {
-        let mut r = ring![1, 2, 3];
+    fn test_iter_copyable() {
+        let r = ring![1, 2, 3];
         let mut iter = r.into_iter();
         assert_eq!(iter.next(), Some(1));
         assert_eq!(iter.next(), Some(2));
         assert_eq!(iter.next(), Some(3));
         assert_eq!(iter.next(), None);
+
+        assert_eq!(r.len(), 3);
+    }
+
+    #[test]
+    fn test_iter_nocopyable() {
+        let mut r = ring![vec![1, 2], vec![3, 4]];
+        let mut iter = r.into_iter();
+        assert_eq!(iter.next(), Some(vec![1, 2]));
+        assert_eq!(iter.next(), Some(vec![3, 4]));
+        assert_eq!(iter.next(), None);
+
+        assert_eq!(r.is_empty(), true);
     }
 
     #[test]
     fn test_collect() {
         let r = ring![1, 2, 3];
-        assert_eq!(vec![1, 2, 3], r.collect());
+        assert_eq!(vec![1, 2, 3], r.collect_vec());
     }
 
     #[test]
     fn test_from_iter() {
         let r = Ring::from_iter(vec![1, 2, 3]);
-        assert_eq!(vec![1, 2, 3], r.collect());
+        assert_eq!(vec![1, 2, 3], r.collect_vec());
     }
 
     #[test]
     fn test_peek() {
         let r = ring![1, 2, 3, 4];
-        assert_eq!(r.peek(0), Some(1));
-        assert_eq!(r.peek(1), Some(2));
-        assert_eq!(r.peek(4), Some(1));
-        assert_eq!(r.peek(-1), Some(4));
-        assert_eq!(r.peek(-1), Some(4));
-        assert_eq!(r.peek(-6), Some(3));
+        assert_eq!(r.peek(0), Some(&1));
+        assert_eq!(r.peek(1), Some(&2));
+        assert_eq!(r.peek(4), Some(&1));
+        assert_eq!(r.peek(-1), Some(&4));
+        assert_eq!(r.peek(-1), Some(&4));
+        assert_eq!(r.peek(-6), Some(&3));
     }
 
     #[test]
     fn test_rotate() {
         let mut r1 = ring![1, 2, 3, 4];
-        assert_eq!(r1.rotate(1).collect(), vec![4, 1, 2, 3]);
+        assert_eq!(r1.rotate(1).collect_vec(), vec![4, 1, 2, 3]);
 
         let mut r2 = ring![1, 2, 3, 4];
-        assert_eq!(r2.rotate(0).collect(), vec![1, 2, 3, 4]);
+        assert_eq!(r2.rotate(0).collect_vec(), vec![1, 2, 3, 4]);
 
         let mut r3 = ring![1, 2, 3, 4];
-        assert_eq!(r3.rotate(-1).collect(), vec![2, 3, 4, 1]);
+        assert_eq!(r3.rotate(-1).collect_vec(), vec![2, 3, 4, 1]);
     }
 
     #[test]
@@ -749,7 +880,7 @@ mod tests {
 
         assert_eq!(r.pop(), Some(4));
         assert_eq!(r.pop_left(), Some(1));
-        assert_eq!(r.collect(), vec![2, 3]);
+        assert_eq!(r.collect_vec(), vec![2, 3]);
     }
 
     #[test]
@@ -791,24 +922,24 @@ mod tests {
         assert_eq!(r.items[1].val, Some(6));
         r.push_left(7);
 
-        assert_eq!(r.collect(), vec![7, 3, 4, 5, 1, 6]);
+        assert_eq!(r.collect_vec(), vec![7, 3, 4, 5, 1, 6]);
     }
 
     #[test]
     fn test_append() {
         let mut r = ring![1, 2, 3];
         r.append(&mut ring![4, 5, 6]);
-        assert_eq!(r.collect(), vec![1, 2, 3, 4, 5, 6]);
+        assert_eq!(r.collect_vec(), vec![1, 2, 3, 4, 5, 6]);
 
         r.append_left(&mut ring![7, 8, 9]);
-        assert_eq!(r.collect(), vec![9, 8, 7, 1, 2, 3, 4, 5, 6]);
+        assert_eq!(r.collect_vec(), vec![9, 8, 7, 1, 2, 3, 4, 5, 6]);
     }
 
     #[test]
     fn test_reverse() {
         let mut r = ring![1, 2, 3];
         r.reverse();
-        assert_eq!(r.collect(), vec![3, 2, 1]);
+        assert_eq!(r.collect_vec(), vec![3, 2, 1]);
     }
 
     #[test]
@@ -822,4 +953,31 @@ mod tests {
         r.clear();
         assert_eq!(r.is_empty(), true);
     }
+
+    #[test]
+    fn test_macro_nocopy() {
+        let r = ring![vec![1, 2, 3]];
+        assert_eq!(r.is_empty(), false);
+    }
+
+    #[test]
+    fn test_eq() {
+        let r = ring![1, 2, 3];
+
+        assert_eq!(r == ring![1, 2], false);
+        assert_eq!(r == ring![1, 2, 3], true);
+        assert_eq!(r == ring![], false);
+
+        assert_eq!(r != ring![1, 2], true);
+        assert_eq!(r != ring![1, 2, 3], false);
+        assert_eq!(r != ring![], true);
+    }
+
+    #[test]
+    fn test_index() {
+        let r = ring![1, 2, 3];
+        assert_eq!(r[0], 1);
+        assert_eq!(r[1], 2);
+        assert_eq!(r[-1], 3);
+    }
 }
